@@ -1,6 +1,6 @@
 /*
  *
- *   Copyright (C) 2012-2015 by C.H. Huang
+ *   Copyright (C) 2012-2016 by C.H. Huang
  *   plushuang.tw@gmail.com
  *
  *  This library is free software; you can redistribute it and/or
@@ -90,17 +90,17 @@ void  uget_app_init (UgetApp* app)
 	app->sorted.notification = &notification_sorted;
 	app->mix.notification = &notification_mix;
 	app->mix_split.notification = &notification_mix_split;
-
 	// add virtual category - "All Category"
 	node = uget_node_new (NULL);
 	node->name = ug_strdup (_("All Category"));
 	uget_node_append (&app->mix, node);
 
+	uget_task_init (&app->task);
+	ug_array_init (&app->nodes, sizeof (void*), 32);
+
 	// plug-in registry
 	app->plugin_default = NULL;
 	ug_registry_init (&app->plugins);
-	uget_task_init (&app->task);
-
 	// info registry
 	ug_registry_init (&app->infos);
 	ug_registry_add (&app->infos, UgetCommonInfo);
@@ -117,6 +117,7 @@ void  uget_app_init (UgetApp* app)
 
 void  uget_app_final (UgetApp* app)
 {
+	ug_array_clear (&app->nodes);
 	uget_task_final (&app->task);
 	uget_app_clear_plugins (app);
 
@@ -133,21 +134,43 @@ void  uget_app_final (UgetApp* app)
 	app->uri_hash = NULL;
 }
 
+static UgArrayPtr* uget_app_store_nodes (UgetApp* app, UgetNode* parent)
+{
+	UgetNode*   dnode;
+	UgArrayPtr* array;
+	int         index;
+
+	array = &app->nodes;
+//	array->length = 0;
+	ug_array_alloc (array, parent->n_children);
+	for (index = 0, dnode = parent->children;  dnode;  dnode = dnode->next)
+		array->at[index++] = dnode->data;
+
+	return array;
+}
+
+static void     uget_app_clear_nodes (UgetApp* app)
+{
+	app->nodes.length = 0;
+}
+
 static int  uget_app_activate (UgetApp* app, UgetNode* cnode, UgetCategory* category)
 {
-	UgetNode*  dnode;
-	UgetNode*  sibling;
-	UgetNode*  dfake;
-	UgetNode*  dfake_next;
-	UgetLog*   log;
+	UgetNode*   dnode;
+	UgetNode*   sibling;
+	UgetLog*    log;
+	UgArrayPtr* array;
+	int         index;
 
-	for (dfake = category->active->children;  dfake;  dfake = dfake_next) {
-		dfake_next = dfake->next;
-		dnode = dfake->real;
+	// Because this function will change node linking,
+	// program must store active nodes to array.
+	array = uget_app_store_nodes (app, category->active);
 
+	for (index = 0;  index < array->length;  index++) {
+		dnode = array->at[index];
 		uget_node_updated (dnode);
 		if (dnode->state & UGET_STATE_ACTIVE) {
-			// remove and insert to resort node
+			// remove node and insert it again to sort node
 			if (app->mix.notification->compare) {
 				sibling = dnode->next;
 				uget_node_remove (cnode, dnode);
@@ -188,19 +211,22 @@ static int  uget_app_activate (UgetApp* app, UgetNode* cnode, UgetCategory* cate
 		app->n_moved++;
 	}
 
+	uget_app_clear_nodes (app);    // clear stored nodes
 	return category->active->n_children;
 }
 
 static void uget_app_queuing (UgetApp* app, UgetNode* cnode, UgetCategory* category)
 {
-	UgetNode*  dnode;
-	UgetNode*  dfake;
-	UgetNode*  dfake_next;
+	UgetNode*   dnode;
+	UgArrayPtr* array;
+	int         index;
 
-	for (dfake = category->queuing->children;  dfake;  dfake = dfake_next) {
-		dfake_next = dfake->next;
-		dnode = dfake->real;
+	// Because uget_app_activate_download() will change node linking,
+	// program must store queuing nodes to array before calling uget_app_activate_download()
+	array = uget_app_store_nodes (app, category->queuing);
 
+	for (index = 0;  index < array->length;  index++) {
+		dnode = array->at[index];
 		if (category->active->n_children >= category->active_limit)
 			break;
 		if (dnode->state & UGET_STATE_INACTIVE)
@@ -208,6 +234,8 @@ static void uget_app_queuing (UgetApp* app, UgetNode* cnode, UgetCategory* categ
 		uget_app_activate_download (app, dnode);
 		app->n_moved++;
 	}
+
+	uget_app_clear_nodes (app);    // clear stored nodes
 }
 
 // return number of active download
@@ -218,7 +246,7 @@ int  uget_app_grow (UgetApp* app, int no_queuing)
 	UgetNode*      dnode;
 	int            n_active = 0;
 
-	// dispatch plugin event, calc speed
+	// dispatch plug-in event, calc speed
 	uget_task_dispatch (&app->task);
 	// active, queuing, finished, recycled
 	for (cnode = app->real.children;  cnode;  cnode = cnode->next) {
@@ -482,13 +510,48 @@ void  uget_app_stop_category (UgetApp* app, UgetNode* cnode)
 {
 	UgetCategory*  category;
 	UgetNode*      dnode;
+	UgArrayPtr*    array;
+	int            index;
 
 	category = ug_info_realloc (&cnode->info, UgetCategoryInfo);
-	for (dnode = category->active->children;  dnode;  dnode = dnode->next)
-		uget_app_queue_download (app, dnode->data);
+
+	// Because uget_app_queue_download() will change node linking,
+	// program must store active nodes to array before calling uget_app_queue_download()
+	array = uget_app_store_nodes (app, category->active);
+
+	for (index = 0;  index < array->length;  index++) {
+		dnode = array->at[index];
+		uget_app_queue_download (app, dnode);
+	}
+
+	uget_app_clear_nodes (app);    // clear stored nodes
 }
 
-UgetNode* uget_app_match_category (UgetApp* app, UgUri* uuri)
+static int  ug_match_file_exts (const char* file, char** exts)
+{
+	const char* beg = NULL;
+	const char* end;
+	int         index;
+
+	// get file ext
+	for (end = file + strlen (file) - 1;  end >= file;  end--) {
+		if (end[0] == '.') {
+			beg = end + 1;   // + '.'
+			break;
+		}
+	}
+
+	if (beg == NULL)
+		return -1;
+
+	for (index = 0;  *exts;  exts++, index++) {
+		if (strcasecmp (beg, *exts) == 0)
+			return index;
+	}
+	return -1;
+}
+
+UgetNode* uget_app_match_category (UgetApp* app, UgUri* uuri, const char* file)
 {
 	UgetCategory* category;
 	UgetNode*     cnode;
@@ -520,6 +583,8 @@ UgetNode* uget_app_match_category (UgetApp* app, UgUri* uuri)
 			count++;
 		if (ug_uri_match_file_exts (uuri, category->file_exts.at) >= 0)
 			count++;
+		else if (file && ug_match_file_exts (file, category->file_exts.at) >= 0)
+			count++;
 
 		if (matched.count < count) {
 			matched.count = count;
@@ -561,6 +626,7 @@ int  uget_app_add_download (UgetApp* app, UgetNode* dnode, UgetNode* cnode, int
 	// decode name, filename, and category
 	if (temp.common->uri) {
 		ug_uri_init (&uuri, temp.common->uri);
+		// assign node name if it's name is NULL
 		if (dnode->name == NULL) {
 			if (temp.common->file)
 				dnode->name = ug_strdup (temp.common->file);
@@ -568,7 +634,7 @@ int  uget_app_add_download (UgetApp* app, UgetNode* dnode, UgetNode* cnode, int
 				uget_node_set_name_by_uri (dnode, &uuri);
 		}
 		if (cnode == NULL)
-			cnode = uget_app_match_category (app, &uuri);
+			cnode = uget_app_match_category (app, &uuri, temp.common->file);
 	}
 
 	if (cnode == NULL)
@@ -662,10 +728,11 @@ int   uget_app_move_download_to (UgetApp* app, UgetNode* dnode, UgetNode* cnode)
 	return TRUE;
 }
 
-void  uget_app_delete_download (UgetApp* app, UgetNode* dnode, int delete_file)
+int  uget_app_delete_download (UgetApp* app, UgetNode* dnode, int delete_file)
 {
 	UgetNode*  fnode;   // filenode
 	UgetNode*  cnode;
+	int        error = 0;
 
 	cnode = dnode->parent;
 	uget_task_remove (&app->task, dnode);
@@ -679,12 +746,17 @@ void  uget_app_delete_download (UgetApp* app, UgetNode* dnode, int delete_file)
 //			if (ug_file_is_exist (fnode->name) == FALSE)
 //				continue;
 			if (ug_file_is_dir (fnode->name) == FALSE)
-				ug_unlink (fnode->name);
+				error = ug_unlink (fnode->name);
 			else
-				ug_delete_dir (fnode->name);
+				error = ug_delete_dir (fnode->name);
 		}
 	}
 	uget_node_unref (dnode);
+
+	if (error == -1)
+		return FALSE;
+	else
+		return TRUE;
 }
 
 int  uget_app_recycle_download (UgetApp* app, UgetNode* dnode)
@@ -727,11 +799,11 @@ int   uget_app_activate_download (UgetApp* app, UgetNode* dnode)
 
 	if (dnode->state & UGET_STATE_ACTIVE)
 		return FALSE;
-	// match plugin
+	// match plug-in
 	log = ug_info_realloc (&dnode->info, UgetLogInfo);
-	temp.pinfo = uget_app_match_plugin (app, dnode);
+	temp.pinfo = uget_app_match_plugin (app, dnode, NULL);
 	if (temp.pinfo == NULL) {
-		// no plugin support
+		// no plug-in support
 		uget_app_queue_download (app, dnode);
 		dnode->state |= UGET_STATE_ERROR;
 		ug_list_prepend (&log->messages,
@@ -748,10 +820,10 @@ int   uget_app_activate_download (UgetApp* app, UgetNode* dnode)
 		log->messages.head = NULL;
 		log->messages.tail = NULL;
 	}
-	// start node with plugin
+	// start node with plug-in
 	cnode = dnode->parent;
 	if (uget_task_add (&app->task, dnode, temp.pinfo) == FALSE) {
-		// plugin start failed.
+		// plug-in start failed.
 		uget_app_queue_download (app, dnode);
 		dnode->state |=  UGET_STATE_ERROR;
 		uget_node_updated (dnode);
@@ -760,9 +832,7 @@ int   uget_app_activate_download (UgetApp* app, UgetNode* dnode)
 	// change node state and move node position
 	uget_node_remove (cnode, dnode);
 	uget_node_unref_fake (dnode);
-	dnode->state &= ~UGET_STATE_UNRUNNABLE;
-	dnode->state &= ~UGET_STATE_CATEGORY;
-	dnode->state |=  UGET_STATE_ACTIVE;
+	dnode->state =  UGET_STATE_ACTIVE;
 	temp.category = ug_info_realloc (&cnode->info, UgetCategoryInfo);
 	// try to insert download before queuing, finished, and recycled
 	sibling = temp.category->queuing->children;
@@ -812,8 +882,14 @@ int   uget_app_pause_download (UgetApp* app, UgetNode* dnode)
 		}
 	}
 #else
-	if (dnode->state & UGET_STATE_ACTIVE)
-		uget_task_remove (&app->task, dnode);
+	if (dnode->state & UGET_STATE_ACTIVE) {
+//		uget_task_remove (&app->task, dnode);
+		UgetRelation*  relation;
+		relation = ug_info_get (&dnode->info, UgetRelationInfo);
+		if (relation && relation->task.plugin)
+			uget_plugin_stop (relation->task.plugin);
+		dnode->state &= ~UGET_STATE_ACTIVE;
+	}
 	else if (dnode->state & UGET_STATE_UNRUNNABLE)
 		return FALSE;
 	dnode->state |= UGET_STATE_PAUSED;
@@ -1009,7 +1085,7 @@ void  uget_app_set_default_plugin (UgetApp* app, const UgetPluginInfo* pinfo)
 	app->plugin_default = (UgetPluginInfo*) pinfo;
 }
 
-UgetPluginInfo*  uget_app_match_plugin (UgetApp* app, UgetNode* node)
+UgetPluginInfo*  uget_app_match_plugin (UgetApp* app, UgetNode* node, const UgetPluginInfo* exclude)
 {
 	UgetCommon*      common;
 	UgetPluginInfo*  info;
@@ -1026,20 +1102,22 @@ UgetPluginInfo*  uget_app_match_plugin (UgetApp* app, UgetNode* node)
 		return NULL;
 
 	ug_uri_init (&uuri, common->uri);
-	if (app->plugin_default) {
+	if (app->plugin_default && app->plugin_default != exclude) {
 		matched.info  = app->plugin_default;
 		matched.count = uget_plugin_match (matched.info, &uuri);
-		if (matched.count == 3)
+		if (matched.count >= 3)
 			return matched.info;
 	}
 	for (index = 0;  index < app->plugins.length;  index++) {
-		info  = app->plugins.at[index].data;
-		count = uget_plugin_match (info, &uuri);
-		if (matched.count < count) {
-			matched.count = count;
-			matched.info  = info;
-			if (matched.count == 3)
-				break;
+		info = app->plugins.at[index].data;
+		if (info != exclude) {
+			count = uget_plugin_match (info, &uuri);
+			if (matched.count < count) {
+				matched.count = count;
+				matched.info  = info;
+				if (matched.count >= 3)
+					break;
+			}
 		}
 	}
 
