@@ -1,6 +1,6 @@
 /*
  *
- *   Copyright (C) 2012-2015 by C.H. Huang
+ *   Copyright (C) 2012-2016 by C.H. Huang
  *   plushuang.tw@gmail.com
  *
  *  This library is free software; you can redistribute it and/or
@@ -207,12 +207,12 @@ static void uget_common_assign (UgetCommon* common, UgetCommon* src)
 
 static const UgEntry  UgetProgressEntry[] =
 {
-	{"complete", offsetof (UgetProgress, complete),    UG_ENTRY_INT64,  NULL, NULL},
-	{"total",    offsetof (UgetProgress, total),       UG_ENTRY_INT64,  NULL, NULL},
-	{"elapsed",  offsetof (UgetProgress, consume_time),UG_ENTRY_INT64,  NULL, NULL},
-	{"uploaded", offsetof (UgetProgress, uploaded),    UG_ENTRY_INT64,  NULL, NULL},
-	{"percent",  offsetof (UgetProgress, percent),     UG_ENTRY_INT,    NULL, NULL},
-//	{"ratio",    offsetof (UgetProgress, ratio),       UG_ENTRY_DOUBLE, NULL, NULL},
+	{"complete", offsetof (UgetProgress, complete), UG_ENTRY_INT64,  NULL, NULL},
+	{"total",    offsetof (UgetProgress, total),    UG_ENTRY_INT64,  NULL, NULL},
+	{"elapsed",  offsetof (UgetProgress, elapsed),  UG_ENTRY_INT64,  NULL, NULL},
+	{"uploaded", offsetof (UgetProgress, uploaded), UG_ENTRY_INT64,  NULL, NULL},
+	{"percent",  offsetof (UgetProgress, percent),  UG_ENTRY_INT,    NULL, NULL},
+//	{"ratio",    offsetof (UgetProgress, ratio),    UG_ENTRY_DOUBLE, NULL, NULL},
 	{NULL}		// null-terminated
 };
 
@@ -240,7 +240,7 @@ static const UgEntry  UgetProxyPwmdEntry[] =
 	{"socket",      offsetof (struct UgetProxyPwmd, socket),
 			UG_ENTRY_STRING, NULL, NULL},
 	{"socket-args", offsetof (struct UgetProxyPwmd, socket_args),
-			UG_TYPE_STRING,	 NULL, NULL},
+			UG_ENTRY_STRING, NULL, NULL},
 	{"file",        offsetof (struct UgetProxyPwmd, file),
 			UG_ENTRY_STRING, NULL, NULL},
 	{"element",     offsetof (struct UgetProxyPwmd, element),
