@@ -1,6 +1,6 @@
 /*
  *
- *   Copyright (C) 2012-2015 by C.H. Huang
+ *   Copyright (C) 2012-2016 by C.H. Huang
  *   plushuang.tw@gmail.com
  *
  *  This library is free software; you can redistribute it and/or
@@ -457,7 +457,8 @@ void  uget_node_set_name_by_uri (UgetNode* node, UgUri* uuri)
 		if (filename) {
 			filename = filename + 3;
 			length = strcspn (filename, "&");
-			node->name = ug_unescape_uri (filename, length);
+			node->name = ug_malloc (length + 1);
+			ug_decode_uri (filename, length, node->name);
 			if (ug_utf8_get_invalid ((uint8_t*) node->name, NULL) != -1) {
 				ug_free (node->name);
 				node->name = ug_strndup (filename, length);
