@@ -1,6 +1,6 @@
 /*
  *
- *   Copyright (C) 2012-2015 by C.H. Huang
+ *   Copyright (C) 2012-2016 by C.H. Huang
  *   plushuang.tw@gmail.com
  *
  *  This library is free software; you can redistribute it and/or
@@ -38,6 +38,14 @@
 #define _CRT_SECURE_NO_WARNINGS
 #endif
 
+#ifdef HAVE_CONFIG_H
+#include <config.h>
+#endif
+
+#ifdef __ANDROID__
+#include <android/api-level.h>
+#endif
+
 #include <UgUtil.h>
 #include <UgStdio.h>
 #include <UgString.h>
@@ -52,6 +60,7 @@
 #include <winsock2.h>
 #define  ug_sleep       Sleep
 #else
+#include <fcntl.h>   // posix_fallocate()
 #include <unistd.h>  // sleep(), usleep()
 #define  ug_sleep(millisecond)    usleep (millisecond * 1000)
 #endif // _WIN32 || _WIN64
@@ -60,6 +69,8 @@
 #define strtoll		_strtoi64    // stdlib.h
 #endif
 
+#define MIN_SPLIT_SIZE       (10 * 1024 * 1024)  // can't less than 16384 x 2
+#define MIN_SPEED_LIMIT      256     // speed control
 #define MAX_REPEAT_DIGITS    5       //  + '.' + digits
 #define MAX_REPEAT_COUNTS    10000   // <= 9999
 
@@ -89,7 +100,6 @@ static UgetResult  global_set (int code, void* parameter);
 static UgetResult  global_get (int code, void* parameter);
 
 static const char* schemes[] = {"http", "https", "ftp", "ftps", NULL};
-static const char* types[]   = {NULL};
 
 static const UgetPluginInfo UgetPluginCurlInfoStatic =
 {
@@ -103,7 +113,7 @@ static const UgetPluginInfo UgetPluginCurlInfoStatic =
 	(UgetPluginSyncFunc) plugin_sync,
 	NULL,
 	schemes,
-	types,
+	NULL,
 	(UgetPluginSetFunc) global_set,
 	(UgetPluginGetFunc) global_get
 };
@@ -119,7 +129,7 @@ static struct
 	int  ref_count;
 } global = {0, 0};
 
-static UgetResult  global_init  (void)
+static UgetResult  global_init (void)
 {
 	if (global.initialized == FALSE) {
 #if defined _WIN32 || defined _WIN64
@@ -194,7 +204,7 @@ static UgetResult  global_get (int option, void* parameter)
 }
 
 // ----------------------------------------------------------------------------
-// plugins functions
+// plug-in functions
 
 static void plugin_init (UgetPluginCurl* plugin)
 {
@@ -203,16 +213,15 @@ static void plugin_init (UgetPluginCurl* plugin)
 	else
 		global_ref ();
 
-	ug_list_init (&plugin->seg.list);
+	ug_list_init (&plugin->segment.list);
 	plugin->file.time = -1;
 	plugin->synced = TRUE;
+	plugin->paused = TRUE;
 	plugin->stopped = TRUE;
 }
 
 static void plugin_final (UgetPluginCurl* plugin)
 {
-	if (plugin->node)
-		uget_node_unref (plugin->node);
 	if (plugin->common)
 		ug_data_free (plugin->common);
 	if (plugin->proxy)
@@ -227,6 +236,9 @@ static void plugin_final (UgetPluginCurl* plugin)
 //	curl_slist_free_all (plugin->ftp_command);
 	ug_free (plugin->file.path);
 	ug_free (plugin->aria2.path);
+	// unassign node
+	if (plugin->node)
+		uget_node_unref (plugin->node);
 
 	global_unref ();
 }
@@ -246,7 +258,7 @@ static int  plugin_ctrl (UgetPluginCurl* plugin, int code, void* data)
 		break;
 
 	case UGET_PLUGIN_CTRL_STOP:
-		plugin->stopped = TRUE;
+		plugin->paused = TRUE;
 		return TRUE;
 
 	case UGET_PLUGIN_CTRL_SPEED:
@@ -292,7 +304,7 @@ static int  plugin_ctrl_speed (UgetPluginCurl* plugin, int* speed)
 		}
 		plugin->limit.upload = value;
 	}
-	// notify plugin that speed limit has been changed
+	// notify plug-in that speed limit has been changed
 	plugin->limit_changed = TRUE;
 	return TRUE;
 }
@@ -315,12 +327,13 @@ static int  plugin_sync (UgetPluginCurl* plugin)
 	if (plugin->stopped) {
 		if (plugin->synced)
 			return FALSE;
-		plugin->synced  = TRUE;
+		plugin->synced = TRUE;
 	}
 	if (plugin->node == NULL)
 		return TRUE;
 
 	node = plugin->node;
+	// sync data between plug-in and node
 	common = ug_info_realloc (&node->info, UgetCommonInfo);
 	common->retry_count = plugin->common->retry_count;
 	// sync changed limit from UgetNode
@@ -331,40 +344,40 @@ static int  plugin_sync (UgetPluginCurl* plugin)
 		plugin->common->max_download_speed = common->max_download_speed;
 		plugin->limit_by_user = TRUE;
 	}
-    plugin->common->max_connections = common->max_connections;
-    plugin->common->retry_limit = common->retry_limit;
-    if (common->max_connections > 0)
-		plugin->seg.n_max = common->max_connections;
+	plugin->common->max_connections = common->max_connections;
+	plugin->common->retry_limit = common->retry_limit;
+	if (common->max_connections > 0)
+		plugin->segment.n_max = common->max_connections;
 
 	progress = ug_info_realloc (&node->info, UgetProgressInfo);
 	progress->upload_speed   = plugin->speed.upload;
 	progress->download_speed = plugin->speed.download;
 
-	if (plugin->size.upload || plugin->size.download) {
-		progress->uploaded   = plugin->size.upload;
-		progress->complete   = plugin->size.download;
+	progress->uploaded   = plugin->size.upload;
+	progress->complete   = plugin->size.download;
 
-		if (plugin->file.size > 0)
-			progress->total = plugin->file.size;
-		else
-			progress->total = progress->complete;
+	if (plugin->file.size > 0)
+		progress->total = plugin->file.size;
+	else
+		progress->total = progress->complete;
 
-		if (progress->total > 0)
-			progress->percent = progress->complete * 100 / progress->total;
-		else
-			progress->percent = 0;
-	}
+	if (progress->total > 0)
+		progress->percent = progress->complete * 100 / progress->total;
+	else
+		progress->percent = 0;
 
 	// If total size and average speed is unknown, don't calculate remain time.
 	if (progress->download_speed > 0 && progress->total > 0) {
-		progress->remain_time = (progress->total - progress->complete) /
+		progress->left = (progress->total - progress->complete) /
 				progress->download_speed;
 	}
-	progress->consume_time = time(NULL) - plugin->start_time;
+	// consume time
+	progress->elapsed = time(NULL) - plugin->start_time;
 
 	// add UgetNode for file & attachment
 	if (plugin->file_renamed && plugin->file.path) {
 		plugin->file_renamed = FALSE;
+		plugin->a2cf_named = FALSE;
 		plugin_clear_node (plugin);
 		plugin_insert_node (plugin, plugin->file.path, FALSE);
 		// change node name
@@ -387,8 +400,8 @@ static int  plugin_sync (UgetPluginCurl* plugin)
 	if (plugin->aria2.path) {
 		if (plugin->file.size == plugin->size.download)
 			plugin_remove_node (plugin, plugin->aria2.path);
-		else if (plugin->aria2_created == FALSE) {
-			plugin->aria2_created = TRUE;
+		else if (plugin->a2cf_named == FALSE) {
+			plugin->a2cf_named = TRUE;
 			plugin_insert_node (plugin, plugin->aria2.path, TRUE);
 		}
 	}
@@ -441,52 +454,59 @@ static void plugin_clear_node (UgetPluginCurl* plugin)
 // ----------------------------------------------------------------------------
 // plugin_start
 
-static void plugin_setup_uris (UgetPluginCurl* plugin);
-static UG_THREAD_RETURN_TYPE plugin_thread (UgetPluginCurl* plugin);
+static void  plugin_setup_uris (UgetPluginCurl* plugin);
+static UG_THREAD_RETURN_TYPE  plugin_thread (UgetPluginCurl* plugin);
 
 static int  plugin_start (UgetPluginCurl* plugin, UgetNode* node)
 {
-	UgThread  thread;
-	int       ok;
-	int       speed[2];
+	UgThread    thread;
+	int         speed[2];
+	union {
+		UgetCommon*  common;
+		UgetProxy*   proxy;
+		UgetHttp*    http;
+		UgetHttp*    ftp;
+		int          ok;
+	} temp;
 
-	plugin->common = ug_info_get (&node->info, UgetCommonInfo);
-	if (plugin->common == NULL || plugin->common->uri == NULL)
-		return  FALSE;
-	plugin->common = ug_data_copy (plugin->common);
+	temp.common = ug_info_get (&node->info, UgetCommonInfo);
+	if (temp.common == NULL || temp.common->uri == NULL)
+		return FALSE;
+	plugin->common = ug_data_copy (temp.common);
 	plugin_setup_uris (plugin);
 
-	plugin->proxy = ug_info_get (&node->info, UgetProxyInfo);
-	if (plugin->proxy)
-		plugin->proxy  = ug_data_copy (plugin->proxy);
+	temp.proxy = ug_info_get (&node->info, UgetProxyInfo);
+	if (temp.proxy)
+		plugin->proxy  = ug_data_copy (temp.proxy);
 
-	plugin->http = ug_info_get (&node->info, UgetHttpInfo);
-	if (plugin->http)
-		plugin->http = ug_data_copy (plugin->http);
-
-	plugin->ftp = ug_info_get (&node->info, UgetFtpInfo);
-	if (plugin->ftp)
-		plugin->ftp = ug_data_copy (plugin->ftp);
-
-	// check http->post_file
-	if (plugin->http->post_file) {
-		if (ug_file_is_exist (plugin->http->post_file) == FALSE) {
-			uget_plugin_post ((UgetPlugin*) plugin,
-					uget_event_new_error (UGET_EVENT_ERROR_POST_FILE_NOT_FOUND,
-					                      NULL));
-			return FALSE;
+	temp.http = ug_info_get (&node->info, UgetHttpInfo);
+	if (temp.http) {
+		plugin->http = ug_data_copy (temp.http);
+		// check http->post_file
+		if (temp.http->post_file) {
+			if (ug_file_is_exist (temp.http->post_file) == FALSE) {
+				uget_plugin_post ((UgetPlugin*) plugin,
+						uget_event_new_error (UGET_EVENT_ERROR_POST_FILE_NOT_FOUND,
+						                      NULL));
+				return FALSE;
+			}
 		}
-	}
-	// check http->cookie_file
-	if (plugin->http->cookie_file) {
-		if (ug_file_is_exist (plugin->http->cookie_file) == FALSE) {
-			uget_plugin_post ((UgetPlugin*) plugin,
-					uget_event_new_error (UGET_EVENT_ERROR_COOKIE_FILE_NOT_FOUND,
-					                      NULL));
-			return FALSE;
+		// check http->cookie_file
+		if (temp.http->cookie_file) {
+			if (ug_file_is_exist (temp.http->cookie_file) == FALSE) {
+				uget_plugin_post ((UgetPlugin*) plugin,
+						uget_event_new_error (UGET_EVENT_ERROR_COOKIE_FILE_NOT_FOUND,
+						                      NULL));
+				return FALSE;
+			}
 		}
 	}
 
+	temp.ftp = ug_info_get (&node->info, UgetFtpInfo);
+	if (temp.ftp)
+		plugin->ftp = ug_data_copy (temp.ftp);
+
+	plugin->start_time = time (NULL);
 	// assign node before speed control
 	uget_node_ref (node);
 	plugin->node = node;
@@ -494,17 +514,24 @@ static int  plugin_start (UgetPluginCurl* plugin, UgetNode* node)
 	speed[0] = plugin->limit.download;
 	speed[1] = plugin->limit.upload;
 	plugin_ctrl_speed (plugin, speed);
+	// Don't notify speed limit changed if user set it before plug-in start.
 	plugin->limit_changed = FALSE;
 
 	// try to start thread
+	plugin->paused = FALSE;
 	plugin->stopped = FALSE;
 	uget_plugin_ref ((UgetPlugin*) plugin);
-	ok = ug_thread_create (&thread, (UgThreadFunc) plugin_thread, plugin);
-	if (ok == UG_THREAD_OK)
+	temp.ok = ug_thread_create (&thread, (UgThreadFunc) plugin_thread, plugin);
+	if (temp.ok == UG_THREAD_OK)
 		ug_thread_unjoin (&thread);
 	else {
-		// failed to start thread
+		// failed to start thread -----------------
+		plugin->paused = TRUE;
 		plugin->stopped = TRUE;
+		// don't assign node
+		uget_node_unref (node);
+		plugin->node = NULL;
+		// post error message and decreases the reference count
 		uget_plugin_post ((UgetPlugin*) plugin,
 				uget_event_new_error (UGET_EVENT_ERROR_THREAD_CREATE_FAILED,
 				                      NULL));
@@ -576,23 +603,26 @@ static void plugin_setup_uris (UgetPluginCurl* plugin)
 // ----------------------------------------------------------------------------
 // plugin_thread
 
-#define N_THREAD(plugin)   ((plugin)->seg.list.size - (plugin)->seg.n_recycled)
+#define N_THREAD(plugin)   ((plugin)->segment.list.size)
 
 static void delay_ms (UgetPluginCurl* plugin, int  milliseconds);
 static int  switch_uri (UgetPluginCurl* plugin, UgetCurl* ugcurl, int is_resumable);
 static int  prepare_file (UgetCurl* ugcurl, UgetPluginCurl* plugin);
 static void complete_file (UgetPluginCurl* plugin);
 static int  load_file_info (UgetPluginCurl* plugin);
+static void clear_file_info (UgetPluginCurl* plugin);
+static int  reuse_download (UgetPluginCurl* plugin, UgetCurl* ugcurl, int next_uri);
 static int  split_download (UgetPluginCurl* plugin, UgetCurl* ugcurl);
 static void adjust_speed_limit (UgetPluginCurl* plugin);
 static UgetCurl* create_segment (UgetPluginCurl* plugin);
 
-static UG_THREAD_RETURN_TYPE plugin_thread (UgetPluginCurl* plugin)
+static UG_THREAD_RETURN_TYPE  plugin_thread (UgetPluginCurl* plugin)
 {
 	UgetCommon* common;
 	UgetCurl*   ugcurl;
 	UgetCurl*   ugnext;
 	int         counter;
+	int         n_active_last = 0;
 	struct {
 		int64_t upload;
 		int64_t download;
@@ -600,209 +630,244 @@ static UG_THREAD_RETURN_TYPE plugin_thread (UgetPluginCurl* plugin)
 
 	common = plugin->common;
 	common->retry_count = 0;
-	plugin->seg.n_max = common->max_connections;
-	if (plugin->seg.n_max == 0)
-		plugin->seg.n_max = 1;
+	plugin->segment.n_max = common->max_connections;
+	if (plugin->segment.n_max == 0)
+		plugin->segment.n_max = 1;
 
+	// create new segment and add it to segment.list
 	ugcurl = create_segment (plugin);
 	if (load_file_info (plugin)) {
 		uget_curl_open_file (ugcurl, plugin->file.path);
-		ugcurl->beg = plugin->seg.beg;
+		ugcurl->beg = plugin->segment.beg;
 		uget_a2cf_lack (&plugin->aria2.ctrl,
 		                (uint64_t*) &ugcurl->beg,
 		                (uint64_t*) &ugcurl->end);
-		plugin->seg.beg = ugcurl->end;
+		plugin->segment.beg = ugcurl->end;
+		plugin->synced = FALSE;
 	}
 	else {
+		clear_file_info (plugin);
 		ugcurl->prepare.func = (UgetCurlFunc) prepare_file;
 		ugcurl->prepare.data = plugin;
 		ugcurl->header_store = TRUE;
 	}
-	// add to seg.list
-	ug_list_append (&plugin->seg.list, (void*) ugcurl);
-	//
-	plugin->start_time = time (NULL);
+	ug_list_append (&plugin->segment.list, (void*) ugcurl);
+
+	// start curl
 	uget_curl_run (ugcurl, FALSE);
 
 	// main loop
 	for (counter = 0;  N_THREAD(plugin) > 0;  counter++) {
 		// sleep 0.5 second
 		ug_sleep (500);
-		// reset data, program will count them later
-		plugin->seg.n_active = 0;
+		// reset data, plug-in will count them (in segment loop) later
+		plugin->segment.n_active = 0;
 		size.upload = 0;
 		size.download = 0;
 		speed.upload = 0;
 		speed.download = 0;
 
-		ugcurl = (UgetCurl*)plugin->seg.list.head;
+		// segment loop
+		ugcurl = (UgetCurl*) plugin->segment.list.head;
 		for (;  ugcurl;  ugcurl = ugnext) {
 			ugnext = ugcurl->next;
-			// split download, use with split_download()
+
+			// split download, use these code with split_download()
 			if (ugcurl->split) {
 				if (ugcurl->prev == NULL || ugcurl->prev->end < ugcurl->beg)
 					ugcurl->split = FALSE;
-				// if prev one overwrite.
-				else if (ugcurl->pos < ugcurl->prev->pos) {
+				else if (ugcurl->beg < ugcurl->prev->pos) {
+					// if previous segment overwrite current one.
 					ugcurl->split = FALSE;
-					ugcurl->stopped = TRUE;
+					ugcurl->paused = TRUE;
 					ugcurl->end = ugcurl->beg;
 					ugcurl->pos = ugcurl->beg;
 					ugcurl->size[0] = 0;
 #ifndef NDEBUG
 					if (common->debug_level) {
-						printf ("\n" "overwrite %u KiB\n",
-								(unsigned) ugcurl->beg / 1024);
+						printf ("\n"
+						        "previous segment overwrite at %u KiB\n",
+						        (unsigned) (ugcurl->beg / 1024));
 					}
 #endif
 				}
-				else if (ugcurl->state == UGET_CURL_RUN) {
+				else if (ugcurl->beg < ugcurl->pos) {
+					// If this segment has downloaded data,
+					// plug-in split new segment from previous one.
 					ugcurl->split = FALSE;
 					ugcurl->prev->end = ugcurl->beg;
 					if (ugcurl->prev->pos > ugcurl->beg) {
 						ugcurl->prev->pos = ugcurl->beg;
-						ugcurl->prev->stopped = TRUE;
 						ugcurl->prev->size[0] = ugcurl->prev->pos -
 						                        ugcurl->prev->beg;
 					}
 #ifndef NDEBUG
 					if (common->debug_level) {
-						printf ("\n" "split %u KiB\n",
-								(unsigned) ugcurl->beg / 1024);
+						printf ("\n"
+						        "split new segment at %u KiB\n",
+						        (unsigned) (ugcurl->beg / 1024));
 					}
 #endif
 				}
 			}
-			// if user want to stop plugin, it must stop all UgetCurl in list.
-			if (plugin->stopped) {
-				ugcurl->stopped = TRUE;
-				plugin->seg.n_max = 0;
+
+			// if user want to stop plug-in, it must stop all UgetCurl in list.
+			if (plugin->paused) {
+				ugcurl->paused = TRUE;
+				plugin->segment.n_max = 0;
 			}
-			// update aria2 ctrl file progress
+			// update aria2 control file progress
 			if (plugin->aria2.path)
 				uget_a2cf_fill (&plugin->aria2.ctrl, ugcurl->beg, ugcurl->pos);
 			// progress
-			if (ugcurl->state > UGET_CURL_RUN) {
+			if (ugcurl->state >= UGET_CURL_OK) {
+				// ugcurl has stopped
 				plugin->base.upload += ugcurl->size[1];
 				plugin->base.download += ugcurl->size[0];
 			}
-			else if (ugcurl->state > UGET_CURL_READY) {
+			else if (ugcurl->state == UGET_CURL_RUN) {
 				size.upload += ugcurl->size[1];
 				size.download += ugcurl->size[0];
 				speed.upload += ugcurl->speed[1];
 				speed.download += ugcurl->speed[0];
 			}
+
 			// handle UgetCurl by state
 			switch (ugcurl->state) {
-			case UGET_CURL_RECYCLED:
-			case UGET_CURL_READY:
+			default:
 				break;
 
 			case UGET_CURL_RUN:
-				plugin->seg.n_active++;
+				plugin->segment.n_active++;
 				break;
 
 			case UGET_CURL_OK:
-				ugcurl->state = UGET_CURL_READY;
-				if (split_download (plugin, ugcurl) == FALSE) {
+				ugcurl->state = UGET_CURL_RESPLIT;
+				// special case for unknown file size
+				if (plugin->file.size == 0 && N_THREAD (plugin) == 1) {
+					complete_file (plugin);
 					// delete download
-					ug_list_remove (&plugin->seg.list, (void*)ugcurl);
+					ug_list_remove (&plugin->segment.list, (void*)ugcurl);
 					uget_curl_free (ugcurl);
 				}
-				if (plugin->file.size == 0 && N_THREAD (plugin) == 0)
-					complete_file (plugin);
+				break;
+
+			case UGET_CURL_ABORT:
+				// delete download
+				ug_list_remove (&plugin->segment.list, (void*)ugcurl);
+				uget_curl_free (ugcurl);
 				break;
 
 			case UGET_CURL_ERROR:
-#if 1
-				if (ugcurl->event_code == UGET_EVENT_ERROR_CONNECT_FAILED)
-					switch_uri (plugin, ugcurl, FALSE);
-#else
-				if (ugcurl->event_code == UGET_EVENT_ERROR_CONNECT_FAILED) {
-					if (switch_uri (plugin, ugcurl, FALSE)) {
-						uget_curl_run (ugcurl, FALSE);
-						break;
-					}
-				}
-#endif
-				// recycle current UgetCurl
-				ugcurl->state = UGET_CURL_RECYCLED;
-				plugin->seg.n_recycled++;
-				if (N_THREAD (plugin) == 0) {
+				// if no other downloading segment, plug-in response error
+				if (N_THREAD (plugin) == 1) {
 //					plugin->node->state |= UGET_STATE_ERROR;
 					if (ugcurl->event) {
 						uget_plugin_post ((UgetPlugin*) plugin, ugcurl->event);
 						ugcurl->event = NULL;
 					}
+					// delete download
+					ug_list_remove (&plugin->segment.list, (void*)ugcurl);
+					uget_curl_free (ugcurl);
+				}
+				else {
+					// try to reuse download
+					reuse_download (plugin, ugcurl, TRUE);
 				}
-				break;
-
-			case UGET_CURL_ABORT:
-				ugcurl->state = UGET_CURL_READY;
-				// delete download
-				ug_list_remove (&plugin->seg.list, (void*)ugcurl);
-				uget_curl_free (ugcurl);
-				// if no download in list, save aria2 ctrl file
-				if (N_THREAD (plugin) == 0 && plugin->aria2.path)
-					uget_a2cf_save (&plugin->aria2.ctrl, plugin->aria2.path);
 				break;
 
 			case UGET_CURL_RETRY:
-				ugcurl->state = UGET_CURL_READY;
-				if (N_THREAD (plugin) == 1)
+				// if no other downloading segment
+				if (N_THREAD (plugin) == 1) {
 					common->retry_count++;
-				if (common->retry_count < common->retry_limit || common->retry_limit == 0) {
-					ugcurl->beg = ugcurl->pos;
-					delay_ms (plugin, common->retry_delay * 1000);
-					uget_curl_run (ugcurl, FALSE);
-					break;
+					if (common->retry_count < common->retry_limit ||
+					    common->retry_limit == 0)
+					{
+						ugcurl->beg = ugcurl->pos;
+						delay_ms (plugin, common->retry_delay * 1000);
+						uget_curl_run (ugcurl, FALSE);
+					}
+					else {
+						// delete segment
+						ug_list_remove (&plugin->segment.list, (void*)ugcurl);
+						uget_curl_free (ugcurl);
+					}
+				}
+				else {
+					// try to reuse download
+					reuse_download (plugin, ugcurl, FALSE);
 				}
-				// delete download
-				ug_list_remove (&plugin->seg.list, (void*)ugcurl);
-				uget_curl_free (ugcurl);
 				break;
 
-			case UGET_CURL_NOT_RESUMABLE:    // redownload - retry
-				ugcurl->state = UGET_CURL_READY;
-				if (switch_uri (plugin, ugcurl, TRUE)) {
-					uget_curl_run (ugcurl, FALSE);
-					break;
-				}
+			case UGET_CURL_NOT_RESUMABLE:
+				// if no other downloading segment
 				if (N_THREAD (plugin) == 1) {
 					uget_plugin_post ((UgetPlugin*) plugin,
 							uget_event_new_normal (
 									UGET_EVENT_NORMAL_NOT_RESUMABLE, NULL));
 					common->retry_count++;
-					if (common->retry_count < common->retry_limit || common->retry_limit == 0) {
+					if (common->retry_count < common->retry_limit ||
+					    common->retry_limit == 0)
+					{
 						plugin->base.download = 0;
+						plugin->size.download = 0;
 						ugcurl->beg = 0;
 						ugcurl->end = plugin->file.size;
 						delay_ms (plugin, common->retry_delay * 1000);
+						switch_uri (plugin, ugcurl, TRUE);
 						uget_curl_run (ugcurl, FALSE);
-						break;
+					}
+					else {
+						// delete download
+						ug_list_remove (&plugin->segment.list, (void*)ugcurl);
+						uget_curl_free (ugcurl);
 					}
 				}
-				// delete download
-				ug_list_remove (&plugin->seg.list, (void*)ugcurl);
-				uget_curl_free (ugcurl);
+				else {
+					// try to reuse download
+					reuse_download (plugin, ugcurl, TRUE);
+				}
 				break;
 			}
 		}
 
-		// progress
+		// use completed UgetCurl to split new segment after segment loop
+		ugcurl = (UgetCurl*) plugin->segment.list.head;
+		for (;  ugcurl;  ugcurl = ugnext) {
+			ugnext = ugcurl->next;
+			if (ugcurl->state == UGET_CURL_RESPLIT) {
+				if (split_download (plugin, ugcurl) == FALSE) {
+					// delete download
+					ug_list_remove (&plugin->segment.list, (void*)ugcurl);
+					uget_curl_free (ugcurl);
+				}
+			}
+		}
+		// progress ---------------------
 		plugin->size.upload = plugin->base.upload + size.upload;
 		plugin->size.download = plugin->base.download + size.download;
-		if (plugin->seg.list.size) {
-			// Don't update speed when stopping
+		// Don't update speed when stopping
+		if (plugin->segment.list.size) {
 			plugin->speed.upload = speed.upload;
 			plugin->speed.download = speed.download;
 		}
 		plugin->synced = FALSE;
-		// check file size
+		// check file size --------------
 		if (plugin->file.size) {
 			// response error if file size is different
 			if (plugin->file.size < plugin->size.download) {
-				plugin->stopped = TRUE;
+#if 0
+
+#ifndef NDEBUG
+				if (common->debug_level) {
+					printf ("file size is different\n");
+					printf ("plugin->file.size = %d\n",     (int)plugin->file.size);
+					printf ("plugin->size.download = %d\n", (int)plugin->size.download);
+				}
+#endif  // NDEBUG
+				plugin->size.download = plugin->file.size;
+#else
+				plugin->paused = TRUE;
 				if (N_THREAD (plugin) > 0)
 					continue;    // wait other thread
 				else {
@@ -815,6 +880,7 @@ static UG_THREAD_RETURN_TYPE plugin_thread (UgetPluginCurl* plugin)
 					plugin->synced = FALSE;
 					break;
 				}
+#endif
 			}
 			// download completed
 			if (plugin->file.size == plugin->size.download) {
@@ -823,40 +889,52 @@ static UG_THREAD_RETURN_TYPE plugin_thread (UgetPluginCurl* plugin)
 				else {
 					complete_file (plugin);
 					plugin->synced = FALSE;
-					plugin->stopped = TRUE;
 					break;
 				}
 			}
 		}
+		// timer ------------------------
 		// adjust speed every 0.5 x 2 = 1 second.
-		if (counter & 1)
+		if ((counter & 1) == 1 || n_active_last != plugin->segment.n_active) {
+			n_active_last = plugin->segment.n_active;
 			adjust_speed_limit (plugin);
-		// save aria2 ctrl file every 0.5 x 4 = 2 seconds.
-		if ((counter & 3) == 3 && plugin->aria2.path)
-			uget_a2cf_save (&plugin->aria2.ctrl, plugin->aria2.path);
+		}
+		// save aria2 control file every 0.5 x 4 = 2 seconds.
+		if ((counter & 3) == 3 || N_THREAD (plugin) == 0) {
+			if (plugin->aria2.path)
+				uget_a2cf_save (&plugin->aria2.ctrl, plugin->aria2.path);
+		}
 		// split download every 0.5 x 8 = 4 seconds.
 		if ((counter & 7) == 7 && plugin->file.size) {
 			// If some threads are connecting, It doesn't split new segment.
-			if (N_THREAD (plugin) <  plugin->seg.n_max &&
-			    N_THREAD (plugin) == plugin->seg.n_active)
+			if (N_THREAD (plugin) <  plugin->segment.n_max &&
+			    N_THREAD (plugin) == plugin->segment.n_active)
 			{
 				split_download (plugin, NULL);
 			}
 		}
-		// retry
+		// retry ------------------------
 		if (common->retry_count >= common->retry_limit && common->retry_limit != 0) {
 			uget_plugin_post ((UgetPlugin*) plugin,
 					uget_event_new_error (
 							UGET_EVENT_ERROR_TOO_MANY_RETRIES, NULL));
 			plugin->synced = FALSE;
-			plugin->stopped = TRUE;
+			plugin->paused = TRUE;
 		}
 	}
 
-	plugin->stopped = TRUE;
-	ug_list_foreach (&plugin->seg.list, (UgForeachFunc) uget_curl_free, NULL);
-	ug_list_init (&plugin->seg.list);
+	// count the latest downloaded size if download doesn't complete
+	if ((plugin->file.size != plugin->size.download) && plugin->aria2.path) {
+		plugin->size.download = uget_a2cf_completed (&plugin->aria2.ctrl);
+		plugin->synced = FALSE;
+	}
+
+	// free segment list
+	ug_list_foreach (&plugin->segment.list, (UgForeachFunc) uget_curl_free, NULL);
+	ug_list_clear (&plugin->segment.list, FALSE);
+	//
 	uget_a2cf_clear (&plugin->aria2.ctrl);
+	plugin->stopped = TRUE;
 	uget_plugin_unref ((UgetPlugin*) plugin);
 	return UG_THREAD_RETURN_VALUE;
 }
@@ -876,17 +954,13 @@ static int prepare_existed (UgetCurl* ugcurl, UgetPluginCurl* plugin)
 			if (plugin->prepared == FALSE) {
 				// plugin_thread() has initialized/created some data for this function.
 				// program must clear these data before calling prepare_file()
-				ug_free (plugin->aria2.path);
-				plugin->aria2.path = NULL;
+				clear_file_info (plugin);
 				uget_curl_close_file (ugcurl);
-				uget_a2cf_clear (&plugin->aria2.ctrl);
 				return prepare_file (ugcurl, plugin);
 			}
 			// don't write INCORRECT data to existed file.
 			ugcurl->event_code = UGET_EVENT_ERROR_INCORRECT_SOURCE;
 			ugcurl->size[0] = 0;
-			if (plugin->seg.list.size == 1)
-				plugin->base.download = uget_a2cf_completed (&plugin->aria2.ctrl);
 			return FALSE;
 		}
 	}
@@ -932,7 +1006,7 @@ static int prepare_file (UgetCurl* ugcurl, UgetPluginCurl* plugin)
 
 	common = plugin->common;
 	// folder
-	if (common->folder == NULL)
+	if (common->folder == NULL || common->folder[0] == 0)
 		length = 0;
 	else {
 		length = strlen (common->folder);
@@ -959,7 +1033,6 @@ static int prepare_file (UgetCurl* ugcurl, UgetPluginCurl* plugin)
 	length += strlen (common->file);
 
 	// path = folder + filename
-	ug_free (plugin->file.path);
 	//                             length + digits + ".aria2" + '\0'
 	plugin->file.path = ug_malloc (length + MAX_REPEAT_DIGITS + 6 + 1);
 	plugin->file.path[0] = 0;  // you need this line if common->folder is NULL.
@@ -1000,6 +1073,7 @@ static int prepare_file (UgetCurl* ugcurl, UgetPluginCurl* plugin)
 				if (plugin->aria2.ctrl.total_len == plugin->file.size) {
 					plugin->aria2.path = ug_strdup (plugin->file.path);
 					plugin->base.download = uget_a2cf_completed (&plugin->aria2.ctrl);
+					plugin->size.download = plugin->base.download;
 					*(char*) strstr (plugin->file.path + length, ".aria2") = 0;
 					break;
 				}
@@ -1007,15 +1081,36 @@ static int prepare_file (UgetCurl* ugcurl, UgetPluginCurl* plugin)
 			}
 		}
 		else {
+			// reset downloaded size if plug-in decide to create new file.
 			plugin->base.download = 0;
+			plugin->size.download = 0;
+			// allocate disk space if plug-in known file size
 			if (plugin->file.size) {
-				// create an empty file of particular size.
+				// preallocate space for a file.
+#if defined _WIN32 || defined _WIN64
+				LARGE_INTEGER size;
+				HANDLE        handle;
+				handle = (HANDLE) _get_osfhandle (value);
+				size.QuadPart = plugin->file.size;
+				if(SetFilePointerEx (handle ,size, 0, FILE_BEGIN) == FALSE)
+					ugcurl->event_code = UGET_EVENT_ERROR_OUT_OF_RESOURCE;
+				if(SetEndOfFile (handle) == FALSE)
+					ugcurl->event_code = UGET_EVENT_ERROR_OUT_OF_RESOURCE;
+				SetFilePointer (handle, 0, 0, FILE_BEGIN);
+#elif defined HAVE_POSIX_FALLOCATE
+				if (posix_fallocate (value, 0, plugin->file.size) != 0)
+					ugcurl->event_code = UGET_EVENT_ERROR_OUT_OF_RESOURCE;
+#elif defined __ANDROID__ && __ANDROID_API__ >= 20
+				if (posix_fallocate64 (value, 0, plugin->file.size) != 0)
+					ugcurl->event_code = UGET_EVENT_ERROR_OUT_OF_RESOURCE;
+#else
 				if (ug_write (value, "O", 1) == -1)  // begin of file
 					ugcurl->event_code = UGET_EVENT_ERROR_OUT_OF_RESOURCE;
 				if (ug_seek (value, plugin->file.size - 1, SEEK_SET) == -1)
 					ugcurl->event_code = UGET_EVENT_ERROR_OUT_OF_RESOURCE;
 				if (ug_write (value, "X", 1) == -1)  // end of file
 					ugcurl->event_code = UGET_EVENT_ERROR_OUT_OF_RESOURCE;
+#endif // _WIN32 || _WIN64
 				// create aria2 control file if no error
 				if (ugcurl->event_code == 0) {
 					plugin->aria2.path = ug_strdup (plugin->file.path);
@@ -1094,7 +1189,7 @@ static int prepare_file (UgetCurl* ugcurl, UgetPluginCurl* plugin)
 	uget_a2cf_lack (&plugin->aria2.ctrl,
 	                (uint64_t*) &temp.val64,
 	                (uint64_t*) &ugcurl->end);
-	plugin->seg.beg = ugcurl->end;
+	plugin->segment.beg = ugcurl->end;
 	if (ugcurl->beg == temp.val64) {
 		if (uget_curl_open_file (ugcurl, plugin->file.path) == FALSE) {
 			ugcurl->event_code = UGET_EVENT_ERROR_FILE_OPEN_FAILED;
@@ -1124,7 +1219,7 @@ static int  load_file_info (UgetPluginCurl* plugin)
 	if (common == NULL || common->file == NULL)
 		return FALSE;
 	// folder
-	if (common->folder == NULL)
+	if (common->folder == NULL || common->folder[0] == 0)
 		length = 0;
 	else {
 		length = strlen (common->folder);
@@ -1153,12 +1248,13 @@ static int  load_file_info (UgetPluginCurl* plugin)
 		return FALSE;
 	}
 	strcat (path, ".aria2");
-	// aria2 ctrl file
+	// aria2 control file
 	if (uget_a2cf_load (&plugin->aria2.ctrl, path)) {
 		plugin->file.size = plugin->aria2.ctrl.total_len;
 		plugin->file.path = ug_strndup (path, length);
 		plugin->aria2.path = path;
 		plugin->base.download = uget_a2cf_completed (&plugin->aria2.ctrl);
+		plugin->size.download = plugin->base.download;
 		return TRUE;
 	}
 	else {
@@ -1168,55 +1264,19 @@ static int  load_file_info (UgetPluginCurl* plugin)
 	}
 }
 
-#if 0
-static int  switch_uri (UgetPluginCurl* plugin, UgetCurl* ugcurl, int is_resumable)
+static void  clear_file_info (UgetPluginCurl* plugin)
 {
-	UriLink*  uri_link;
-	UriLink*  uri_begin;
-	int       cycled = FALSE;
+	uget_a2cf_clear (&plugin->aria2.ctrl);
+	ug_free (plugin->aria2.path);
+	plugin->aria2.path = NULL;
 
-	if (plugin->uri.link)
-		uri_begin = (UriLink*) plugin->uri.link;
-	else
-		uri_begin = (UriLink*) plugin->uri.list.head;
-
-	for (uri_link = uri_begin;  ;) {
-		if (uri_link == uri_begin && cycled)
-			break;
-		if (uri_link == NULL) {
-			uri_link = (UriLink*) plugin->uri.list.head;
-			cycled = TRUE;
-			continue;
-		}
-		if (uri_link->tested) {
-			if (uri_link->ok == FALSE) {
-				uri_link = uri_link->next;
-				continue;
-			}
-			if (is_resumable && uri_link->resumable == FALSE) {
-				uri_link = uri_link->next;
-				continue;
-			}
-		}
-		break;
-	}
-
-	// set URI and decide it's scheme
-	uget_curl_set_url (ugcurl, uri_link->uri);
-	uri_link->scheme_type = ugcurl->scheme_type;
-	// sync URI flags to UgetCurl
-	ugcurl->uri.link = uri_link;
-	ugcurl->resumable = uri_link->resumable;
-	ugcurl->tested = uri_link->tested;
-	ugcurl->test_ok = uri_link->ok;
-	// pointer current URI to next one
-	plugin->uri.link = (void*) uri_link->next;
-
-	if (uri_link == uri_begin)
-		return FALSE;
-	return TRUE;
+	ug_free (plugin->file.path);
+	plugin->file.path = NULL;
+	plugin->file.size = 0;
+	plugin->base.download = 0;
+	plugin->size.download = 0;
 }
-#else
+
 static int  switch_uri (UgetPluginCurl* plugin, UgetCurl* ugcurl, int is_resumable)
 {
 	UriLink*  uri_link;
@@ -1238,12 +1298,14 @@ static int  switch_uri (UgetPluginCurl* plugin, UgetCurl* ugcurl, int is_resumab
 
 	return TRUE;
 }
-#endif
 
 static void complete_file (UgetPluginCurl* plugin)
 {
-	if (plugin->aria2.path)
+	if (plugin->aria2.path) {
 		ug_unlink (plugin->aria2.path);
+		ug_free (plugin->aria2.path);
+		plugin->aria2.path = NULL;
+	}
 	// modify file time
 	if (plugin->common->timestamp == TRUE && plugin->file.time != -1)
 		ug_modify_file_time (plugin->file.path, plugin->file.time);
@@ -1255,7 +1317,25 @@ static void complete_file (UgetPluginCurl* plugin)
 			uget_event_new (UGET_EVENT_STOP));
 }
 
-static int split_download (UgetPluginCurl* plugin, UgetCurl* ugcurl)
+static int  reuse_download (UgetPluginCurl* plugin, UgetCurl* ugcurl, int next_uri)
+{
+	if (ugcurl->beg == ugcurl->pos) {
+		// delete segment if no downloaded data
+		ug_list_remove (&plugin->segment.list, (void*)ugcurl);
+		uget_curl_free (ugcurl);
+		return FALSE;
+	}
+	else {
+		// reuse this segment
+		if (next_uri == TRUE)
+			switch_uri (plugin, ugcurl, TRUE);
+		ugcurl->beg = ugcurl->pos;
+		uget_curl_run (ugcurl, FALSE);
+		return TRUE;
+	}
+}
+
+static int  split_download (UgetPluginCurl* plugin, UgetCurl* ugcurl)
 {
 	UgetCurl*  temp;
 	UgetCurl*  sibling = NULL;
@@ -1264,19 +1344,11 @@ static int split_download (UgetPluginCurl* plugin, UgetCurl* ugcurl)
 
 	if (plugin->aria2.path == NULL)
 		return FALSE;
-	// find recycled UgetCurl and start it.
-	for (temp = (void*)plugin->seg.list.head;  temp;  temp = temp->next) {
-		if (temp->state == UGET_CURL_RECYCLED) {
-			plugin->seg.n_recycled--;
-			switch_uri (plugin, temp, TRUE);
-			uget_curl_run (temp, FALSE);
-			return FALSE;
-		}
-	}
+
 	// try to find unused space
-	cur = plugin->seg.beg;
+	cur = plugin->segment.beg;
 	if (uget_a2cf_lack (&plugin->aria2.ctrl, &cur, &end)) {
-		plugin->seg.beg = end;
+		plugin->segment.beg = end;
 #ifndef NDEBUG
 		if (plugin->common->debug_level) {
 			printf ("\n" "lack %u-%u KiB\n",
@@ -1285,10 +1357,11 @@ static int split_download (UgetPluginCurl* plugin, UgetCurl* ugcurl)
 		}
 #endif
 	}
+	// if no unused space, try to split downloading segment.
 	else {
-		// if no unused space, try to split current download
+		// cur = segment size;  end = the largest segment size;
 		end = 0;
-		for (temp = (void*)plugin->seg.list.head;  temp;  temp = temp->next) {
+		for (temp = (void*)plugin->segment.list.head;  temp;  temp = temp->next) {
 			cur = temp->end - temp->pos;
 			if (end < cur) {
 				end = cur;
@@ -1297,10 +1370,11 @@ static int split_download (UgetPluginCurl* plugin, UgetCurl* ugcurl)
 		}
 		if (sibling == NULL)
 			return FALSE;
-		//
 		cur = (sibling->end - sibling->pos) >> 1;
-		if (cur < 16384 * 3)
+		// if segment is too small, don't split it.
+		if (cur < MIN_SPLIT_SIZE)
 			return FALSE;
+		// cur = begin of new segment;  end = end of new segment;
 		cur = sibling->end - cur;
 		end = sibling->end;
 		if (cur & 16383)
@@ -1316,18 +1390,18 @@ static int split_download (UgetPluginCurl* plugin, UgetCurl* ugcurl)
 	}
 
 	// reuse or create UgetCurl
-	// if this UgetCurl has been inserted in seg.list, remove it.
+	// if this UgetCurl has been inserted in segment.list, remove it.
 	if (ugcurl)
-		ug_list_remove (&plugin->seg.list, (UgLink*) ugcurl);
+		ug_list_remove (&plugin->segment.list, (UgLink*) ugcurl);
 	else
 		ugcurl = create_segment (plugin);
 
-	// add to seg.list
+	// add to segment.list
 	if (sibling == NULL)
-		ug_list_append (&plugin->seg.list, (void*) ugcurl);
+		ug_list_append (&plugin->segment.list, (void*) ugcurl);
 	else {
 		ugcurl->split = TRUE;
-		ug_list_insert (&plugin->seg.list,
+		ug_list_insert (&plugin->segment.list,
 				(void*) sibling->next, (void*) ugcurl);
 	}
 
@@ -1339,7 +1413,7 @@ static int split_download (UgetPluginCurl* plugin, UgetCurl* ugcurl)
 
 static void delay_ms (UgetPluginCurl* plugin, int  milliseconds)
 {
-	while (plugin->stopped == FALSE) {
+	while (plugin->paused == FALSE) {
 		if (milliseconds >  500) {
 			milliseconds -= 500;
 			ug_sleep (500);
@@ -1359,6 +1433,11 @@ static UgetCurl* create_segment (UgetPluginCurl* plugin)
 	uget_curl_set_proxy (ugcurl, plugin->proxy);
 	uget_curl_set_http (ugcurl, plugin->http);
 	uget_curl_set_ftp (ugcurl, plugin->ftp);
+	// set speed limit
+	if (plugin->limit.download)
+		ugcurl->limit[0] = plugin->limit.download / (plugin->segment.list.size + 1);
+	if (plugin->limit.upload)
+		ugcurl->limit[1] = plugin->limit.upload / (plugin->segment.list.size + 1);
 	// select URL
 	switch_uri (plugin, ugcurl, FALSE);
 	// set output function
@@ -1368,38 +1447,37 @@ static UgetCurl* create_segment (UgetPluginCurl* plugin)
 }
 
 // speed control
-#define SPEED_MIN    256
-
 static void  adjust_speed_limit_index (UgetPluginCurl* plugin, int idx, int64_t remain)
 {
 	UgetCurl*  ucurl;
 
 	// balance speed
-	remain = remain / plugin->seg.n_active;
+	remain = remain / plugin->segment.n_active;
 
-	for (ucurl = (UgetCurl*) plugin->seg.list.head; ucurl; ucurl=ucurl->next) {
+	for (ucurl = (UgetCurl*) plugin->segment.list.head; ucurl; ucurl=ucurl->next) {
 		if (ucurl->state != UGET_CURL_RUN)
 			continue;
 		ucurl->limit[idx] = ucurl->speed[idx] + remain;
-		if (ucurl->limit[idx] < SPEED_MIN)
-			ucurl->limit[idx] = SPEED_MIN;
+		if (ucurl->limit[idx] < MIN_SPEED_LIMIT)
+			ucurl->limit[idx] = MIN_SPEED_LIMIT;
 		ucurl->limit_changed = TRUE;
 	}
 }
 
 static void  disable_speed_limit (UgetPluginCurl* plugin, int idx)
 {
-	UgetCurl*  ucurl;
+	UgetCurl*  ugcurl;
 
-	for (ucurl = (UgetCurl*) plugin->seg.list.head; ucurl; ucurl=ucurl->next) {
-		ucurl->limit[idx] = 0;
-		ucurl->limit_changed = TRUE;
+	ugcurl = (UgetCurl*) plugin->segment.list.head;
+	for (;  ugcurl;  ugcurl = ugcurl->next) {
+		ugcurl->limit[idx] = 0;
+		ugcurl->limit_changed = TRUE;
 	}
 }
 
 static void  adjust_speed_limit (UgetPluginCurl* plugin)
 {
-	if (plugin->seg.n_active == 0)
+	if (plugin->segment.n_active == 0)
 		return;
 
 	// download
