@@ -1,6 +1,6 @@
 /*
  *
- *   Copyright (C) 2012-2015 by C.H. Huang
+ *   Copyright (C) 2012-2016 by C.H. Huang
  *   plushuang.tw@gmail.com
  *
  *  This library is free software; you can redistribute it and/or
@@ -42,7 +42,7 @@
 #include <UgetData.h>
 #include <UgetPlugin.h>
 #include <UgetA2cf.h>
-#include <curl/curl.h>
+//#include <curl/curl.h>    // curl_slist
 
 #ifdef __cplusplus
 extern "C" {
@@ -53,18 +53,20 @@ typedef struct UgetPluginCurl     UgetPluginCurl;
 extern const UgetPluginInfo*  UgetPluginCurlInfo;
 
 // ----------------------------------------------------------------------------
-// UgetPluginCurl: an empty plugin. It derived from UgetPlugin.
+// UgetPluginCurl: libcurl plug-in that derived from UgetPlugin.
 
 struct UgetPluginCurl
 {
-	UGET_PLUGIN_MEMBERS;               // It derived from UgetPlugin
+	UGET_PLUGIN_MEMBERS;           // It derived from UgetPlugin
 //	const UgetPluginInfo*  info;
 //	UgetEvent*    messages;
 //	UgMutex       mutex;
 //	int           ref_count;
 
-	// UgetNode data
+	// pointer to UgetNode that store in UgetApp
 	UgetNode*     node;
+
+	// copy of UgetNode data
 	UgetCommon*   common;
 	UgetProxy*    proxy;
 	UgetHttp*     http;
@@ -78,7 +80,7 @@ struct UgetPluginCurl
 		int64_t   size;        // total size (0 if size unknown)
 	} file;
 
-	// aria2 ctrl file
+	// aria2 control file
 	struct {
 		char*     path;
 		UgetA2cf  ctrl;
@@ -92,20 +94,19 @@ struct UgetPluginCurl
 
 	// segment (split download)
 	struct {
-		UgList    list;    // list of UgetCurl
-		int64_t   beg;
-		uintptr_t n_max;
-		uintptr_t n_active;
-		uintptr_t n_recycled;
-	} seg;
+		UgList    list;    // list of segment (UgetCurl)
+		int64_t   beg;     // beginning of undownloaded position
+		int       n_max;
+		int       n_active;
+	} segment;
 
 	// progress for uget_plugin_sync()
 	time_t        start_time;
 
-	// base.download = base downloaded size
-	// base.upload = base uploaded size
-	// size.download = downloaded size
-	// size.upload = uploaded size
+	// base.download = base downloaded size  (existing downloaded size)
+	// base.upload = base uploaded size      (existing uploaded size)
+	// size.download = downloaded size  (base + threads downloaded size)
+	// size.upload = uploaded size      (base + threads uploaded size)
 	// speed.download = downloading speed
 	// speed.upload = uploading speed
 	// limit.download = download speed limit
@@ -119,9 +120,10 @@ struct UgetPluginCurl
 	uint8_t       limit_by_user:1; // speed limit changed by user
 	uint8_t       limit_changed:1; // speed limit changed
 	uint8_t       file_renamed:1;  // has file path?
-	uint8_t       aria2_created:1; // has aria2 ctrl file?
+	uint8_t       a2cf_named:1;    // has aria2 ctrl file name?
 	uint8_t       synced:1;
-	uint8_t       stopped:1;       // ctrl
+	uint8_t       paused:1;        // paused by user or program
+	uint8_t       stopped:1;       // all of downloading thread are stopped
 	uint8_t       prepared:1;      // prepare to download
 };
 
@@ -139,12 +141,10 @@ namespace Uget
 
 // This one is for derived use only. No data members here.
 // Your derived struct/class must be C++11 standard-layout
-struct PluginCurlMethod : Uget::PluginMethod
-{
-};
+struct PluginCurlMethod : Uget::PluginMethod {};
 
 // This one is for directly use only. You can NOT derived it.
-struct PluginCurl : Uget::PluginEmptyMethod, UgetPluginCurl {};
+struct PluginCurl : Uget::PluginCurlMethod, UgetPluginCurl {};
 
 };  // namespace Uget
 
