@@ -1,6 +1,6 @@
 /*
  *
- *   Copyright (C) 2005-2015 by C.H. Huang
+ *   Copyright (C) 2005-2016 by C.H. Huang
  *   plushuang.tw@gmail.com
  *
  *  This library is free software; you can redistribute it and/or
@@ -47,11 +47,11 @@ int  ug_uri_init (UgUri* upart, const char* uri)
 	const char* cur;
 	const char* tmp;
 
-	// scheme
+	// scheme - make sure ':' before '/', '%', '?', and '#'
 #if defined _WIN32 || defined _WIN64
-	cur = strpbrk (uri, ":\\/?#");  // make sure ':' before '/', '?', and '#'
+	cur = strpbrk (uri, ":\\/%?#");
 #else
-	cur = strpbrk (uri, ":/?#");    // make sure ':' before '/', '?', and '#'
+	cur = strpbrk (uri, ":/%?#");
 #endif
 	if (cur && cur[0] == ':') {
 		if (upart == NULL)
@@ -117,7 +117,10 @@ int  ug_uri_init (UgUri* upart, const char* uri)
 		upart->fragment = -1;
 	else {
 		cur = tmp + 1;
-		upart->fragment = cur - uri;
+		if (cur - uri <= INT16_MAX)
+			upart->fragment = cur - uri;
+		else
+			upart->fragment = -1;
 	}
 
 	// host & port
@@ -286,7 +289,8 @@ char* ug_uri_get_file (UgUri* uuri)
 	len = ug_uri_part_file (uuri, &str);
 	if (len == 0)
 		return NULL;
-	name = ug_unescape_uri (str, len);
+	name = ug_malloc (len + 1);
+	ug_decode_uri (str, len, name);
 	if (ug_utf8_get_invalid ((uint8_t*) name, NULL) == -1)
 		return name;
 
@@ -361,6 +365,54 @@ int  ug_uri_match_file_exts (UgUri* uuri, char** exts)
 	return -1;
 }
 
+// ------------------------------------
+// UgUriQuery
+
+int  ug_uri_query_part (UgUriQuery* uuquery, const char* query_field)
+{
+	int  ret_value;
+
+	if (query_field == NULL)
+		return 0;
+	uuquery->field_len = strcspn (query_field, "=&#;,");
+	if (uuquery->field_len == 0)
+		return 0;
+
+	query_field += uuquery->field_len;
+	if (query_field[0] == '=') {
+		uuquery->value = (char*) query_field + 1;   // + '='
+		uuquery->value_len = strcspn (uuquery->value, "&#;,");
+		query_field = uuquery->value + uuquery->value_len;
+		ret_value = 2;
+	}
+	else {
+		uuquery->value = NULL;
+		uuquery->value_len = 0;
+		ret_value = 1;
+	}
+
+	// next field or next value
+	switch (query_field[0]) {
+	case ',':
+		uuquery->value_next = (char*) query_field + 1;
+		uuquery->field_next = NULL;
+		break;
+
+	case '&':
+	case ';':
+		uuquery->value_next = NULL;
+		uuquery->field_next = (char*) query_field + 1;
+		break;
+
+	default:
+		uuquery->value_next = NULL;
+		uuquery->field_next = NULL;
+		break;
+	}
+
+	return ret_value;
+}
+
 // ------------------------------------
 // Other URI functions
 
@@ -370,5 +422,44 @@ char* ug_filename_from_uri (const char* str)
 
 	ug_uri_init (&uuri, str);
 	return ug_uri_get_file ((UgUri*) &uuri);
+}
 
+// return length of decoded uri. param dest can be param uri or NULL.
+int   ug_decode_uri (const char* uri, int length, char* dest)
+{
+	const char*  uri_end;
+	char*  dest_beg;
+	char*  endptr;
+	char   buf[3];
+
+	if (length == -1)
+		length = strlen (uri);
+	dest_beg = dest;
+
+	for (uri_end = uri + length;  uri < uri_end;  uri++, dest++) {
+		if (uri[0] == '%' && uri + 2 < uri_end) {
+			buf[0] = uri[1];
+			buf[1] = uri[2];
+			buf[2] = 0;
+			endptr = NULL;
+			*(uint8_t*)buf = (uint8_t) strtoul (buf, &endptr, 16);
+			if (endptr == buf + 2) {
+				if (dest_beg)
+					*dest = *buf;
+				uri += 2;
+				continue;
+			}
+		}
+
+		if (dest_beg) {
+			if (uri[0] == '+')
+				*dest = ' ';
+			else
+				*dest = *uri;
+		}
+	}
+
+	if (dest_beg)
+		*dest = 0;
+	return dest - dest_beg;
 }
