@@ -1,6 +1,6 @@
 /*
  *
- *   Copyright (C) 2005-2015 by C.H. Huang
+ *   Copyright (C) 2005-2016 by C.H. Huang
  *   plushuang.tw@gmail.com
  *
  *  This library is free software; you can redistribute it and/or
@@ -124,9 +124,40 @@ int  ug_uri_match_hosts   (UgUri* uuri, char** hosts);
 int  ug_uri_match_schemes (UgUri* uuri, char** schemes);
 int  ug_uri_match_file_exts (UgUri* uuri, char** exts);
 
+// ------------------------------------
+// UgUriQuery
+
+// field1=value1&field2=value2&field3=value3,next_value
+
+typedef struct UgUriQuery         UgUriQuery;
+
+struct UgUriQuery {
+	int    field_len;
+	char*  value;
+	int    value_len;
+
+	char*  field_next;
+	char*  value_next;
+};
+
+// param query_field can be NULL
+// return 0: no field (end of query)
+// return 1: field only
+// return 2: field & value
+//
+// while (ug_uri_query_part (&uuquery, field) > 0) {
+//	// your code here
+//	field = uuquery->next;
+// }
+//
+int  ug_uri_query_part (UgUriQuery* uuquery, const char* query_field);
+
 // ------------------------------------
 // Other URI functions
 
+// return length of decoded uri. param dest can be param uri or NULL.
+int   ug_decode_uri (const char* uri, int uri_length, char* dest);
+
 char* ug_filename_from_uri (const char* uri);
 
 #ifdef __cplusplus
