@@ -1,6 +1,6 @@
 /*
  *
- *   Copyright (C) 2012-2015 by C.H. Huang
+ *   Copyright (C) 2012-2016 by C.H. Huang
  *   plushuang.tw@gmail.com
  *
  *  This library is free software; you can redistribute it and/or
@@ -65,12 +65,17 @@ void  ugtk_app_init_callback (UgtkApp* app)
 
 // ----------------------------------------------------------------------------
 // Toolbar
-//
+
 static void  on_create_download (GtkWidget* widget, UgtkApp* app)
 {
 	ugtk_app_create_download (app, NULL, NULL);
 }
 
+static void  on_set_download_runnable (GtkWidget* widget, UgtkApp* app)
+{
+	ugtk_app_queue_download (app, TRUE);
+}
+
 // ----------------------------------------------------------------------------
 // UgtkWindow
 
@@ -160,10 +165,10 @@ static void  on_category_cursor_changed (GtkTreeView* view, UgtkApp* app)
 			sensitive = FALSE;
 		else
 			sensitive = TRUE;
-		gtk_widget_set_sensitive (app->menubar.view.columns.completed, sensitive);
+		gtk_widget_set_sensitive (app->menubar.view.columns.complete, sensitive);
 		gtk_widget_set_sensitive (app->menubar.view.columns.percent, sensitive);
 		column = gtk_tree_view_get_column (view, UGTK_NODE_COLUMN_COMPLETE);
-		gtk_tree_view_column_set_visible (column, sensitive && setting->completed);
+		gtk_tree_view_column_set_visible (column, sensitive && setting->complete);
 		column = gtk_tree_view_get_column (view, UGTK_NODE_COLUMN_PERCENT);
 		gtk_tree_view_column_set_visible (column, sensitive && setting->percent);
 		// Recycled
@@ -455,8 +460,8 @@ static void ugtk_toolbar_init_callback (struct UgtkToolbar* toolbar, UgtkApp* ap
 	g_signal_connect_swapped (toolbar->save, "clicked",
 			G_CALLBACK (ugtk_app_save), app);
 	// change status
-	g_signal_connect_swapped (toolbar->runnable, "clicked",
-			G_CALLBACK (ugtk_app_queue_download), app);
+	g_signal_connect (toolbar->runnable, "clicked",
+			G_CALLBACK (on_set_download_runnable), app);
 	g_signal_connect_swapped (toolbar->pause, "clicked",
 			G_CALLBACK (ugtk_app_pause_download), app);
 	// change data
