@@ -1,6 +1,6 @@
 /*
  *
- *   Copyright (C) 2005-2015 by C.H. Huang
+ *   Copyright (C) 2005-2016 by C.H. Huang
  *   plushuang.tw@gmail.com
  *
  *  This library is free software; you can redistribute it and/or
@@ -67,7 +67,9 @@ static void ugtk_app_notify_error (UgtkApp* app);
 static void ugtk_app_notify_starting (UgtkApp* app);
 static void ugtk_app_notify_completed (UgtkApp* app);
 // GSourceFunc
+#ifdef HAVE_RSS_NOTIFY
 static gboolean  ugtk_app_timeout_rss (UgtkApp* app);
+#endif
 static gboolean  ugtk_app_timeout_rpc (UgtkApp* app);
 static gboolean  ugtk_app_timeout_queuing (UgtkApp* app);
 static gboolean  ugtk_app_timeout_clipboard (UgtkApp* app);
@@ -84,9 +86,11 @@ void  ugtk_app_init_timeout (UgtkApp* app)
 	// 2 seconds
 	g_timeout_add_seconds_full (G_PRIORITY_DEFAULT_IDLE, 2,
 			(GSourceFunc) ugtk_app_timeout_clipboard, app, NULL);
+#ifdef HAVE_RSS_NOTIFY
 	// 3 seconds
 	g_timeout_add_seconds_full (G_PRIORITY_DEFAULT_IDLE, 3,
 			(GSourceFunc) ugtk_app_timeout_rss, app, NULL);
+#endif  // HAVE_RSS_NOTIFY
 	// 1 minutes
 	g_timeout_add_seconds_full (G_PRIORITY_DEFAULT_IDLE, 60,
 			(GSourceFunc) ugtk_app_timeout_autosave, app, NULL);
@@ -174,7 +178,7 @@ static gboolean  ugtk_app_decide_schedule_state (UgtkApp* app)
 static gboolean  ugtk_app_timeout_queuing (UgtkApp* app)
 {
 	static int  n_counts = 0;
-	static int  n_active_prev = 0;
+	static int  n_active_last = 0;
 	int         n_active;
 	int         no_queuing = FALSE;
 	gchar*      string;
@@ -194,15 +198,15 @@ static gboolean  ugtk_app_timeout_queuing (UgtkApp* app)
 	}
 
 	n_active = uget_app_grow ((UgetApp*) app, no_queuing);
-	if (n_active != n_active_prev) {
+	if (n_active != n_active_last) {
 		// start downloading
-		if (n_active > 0 && n_active_prev == 0) {
+		if (n_active > 0 && n_active_last == 0) {
 			// starting notification
 			if (app->setting.ui.start_notification)
 				ugtk_app_notify_starting (app);
 		}
 		// stop downloading
-		else if (n_active == 0 && n_active_prev > 0) {
+		else if (n_active == 0 && n_active_last > 0) {
 			if (app->n_error > 0) {
 				// error notification
 				ugtk_app_notify_error (app);
@@ -294,7 +298,7 @@ static gboolean  ugtk_app_timeout_queuing (UgtkApp* app)
 
 	app->user_action = FALSE;
 	app->n_moved = 0;   // reset counter
-	n_active_prev = n_active;
+	n_active_last = n_active;
 	n_counts++;
 	return TRUE;
 }
@@ -309,7 +313,8 @@ static void  on_keep_above_window_show (GtkWindow *window, gpointer  user_data)
 	gtk_window_set_keep_above (window, FALSE);
 }
 
-static void  ugtk_app_add_uris_quietly (UgtkApp* app, GList* list, UgetNode* infonode, int  nth_category)
+static void  ugtk_app_add_uris_quietly (UgtkApp* app,       GList* list,
+                                        UgetNode* infonode, int  nth_category)
 {
 	GList*      link;
 	UgUri       uuri;
@@ -334,14 +339,14 @@ static void  ugtk_app_add_uris_quietly (UgtkApp* app, GList* list, UgetNode* inf
 		if (cnode == NULL) {
 			// match category by URI
 			ug_uri_init (&uuri, link->data);
-			cnode = uget_app_match_category ((UgetApp*) app, &uuri);
+			cnode = uget_app_match_category ((UgetApp*) app, &uuri, NULL);
 		}
 		if (cnode == NULL && infonode) {
 			// match category by filename
 			common = ug_info_realloc (&infonode->info, UgetCommonInfo);
 			if (common && common->file) {
 				ug_uri_init (&uuri, common->file);
-				cnode = uget_app_match_category ((UgetApp*) app, &uuri);
+				cnode = uget_app_match_category ((UgetApp*) app, &uuri, NULL);
 			}
 		}
 		if (cnode == NULL) {
@@ -373,7 +378,8 @@ static void  ugtk_app_add_uris_quietly (UgtkApp* app, GList* list, UgetNode* inf
 	}
 }
 
-static void  ugtk_app_add_uris_selected (UgtkApp* app, GList* list, UgetNode* infonode, int  nth_category)
+static void  ugtk_app_add_uris_selected (UgtkApp* app,       GList* list,
+                                         UgetNode* infonode, int  nth_category)
 {
 	UgtkBatchDialog*    bdialog;
 	UgtkSelectorPage*   page;
@@ -430,14 +436,11 @@ static void  ugtk_app_add_uris_selected (UgtkApp* app, GList* list, UgetNode* in
 	if (cnode == NULL && list->data) {
 		// match category by URI
 		ug_uri_init (&uuri, list->data);
-		cnode = uget_app_match_category ((UgetApp*) app, &uuri);
-	}
-	if (cnode == NULL && infonode) {
-		// match category by filename
-		common = ug_info_realloc (&infonode->info, UgetCommonInfo);
-		if (common && common->file) {
-			ug_uri_init (&uuri, common->file);
-			cnode = uget_app_match_category ((UgetApp*) app, &uuri);
+		if (infonode == NULL)
+			cnode = uget_app_match_category ((UgetApp*) app, &uuri, NULL);
+		else {
+			common = ug_info_realloc (&infonode->info, UgetCommonInfo);
+			cnode = uget_app_match_category ((UgetApp*) app, &uuri, common->file);
 		}
 	}
 	if (cnode == NULL) {
@@ -463,9 +466,9 @@ static void  ugtk_app_add_uris_selected (UgtkApp* app, GList* list, UgetNode* in
 	ugtk_batch_dialog_run (bdialog);
 }
 
-static void on_clipboard_text_received (GtkClipboard*	clipboard,
-                                        const gchar*	text,
-                                        gpointer		user_data)
+static void on_clipboard_text_received (GtkClipboard*  clipboard,
+                                        const gchar*   text,
+                                        gpointer       user_data)
 {
 	UgtkApp*  app;
 	GList*    list;
@@ -573,11 +576,14 @@ static gboolean  ugtk_app_timeout_rpc (UgtkApp* app)
 		}
 		req->free (req);
 	}
+
+	return TRUE;
 }
 
 // ----------------------------------------------------------------------------
 // RSS
 
+#ifdef HAVE_RSS_NOTIFY
 static gboolean  ugtk_app_timeout_rss_update (UgtkApp* app)
 {
 	uget_rss_update (app->rss_builtin, FALSE);
@@ -603,6 +609,7 @@ static gboolean  ugtk_app_timeout_rss (UgtkApp* app)
 	}
 	return TRUE;
 }
+#endif  // HAVE_RSS_NOTIFY
 
 // ----------------------------------------------------------------------------
 // sound
