@@ -1,6 +1,6 @@
 /*
  *
- *   Copyright (C) 2012-2015 by C.H. Huang
+ *   Copyright (C) 2012-2016 by C.H. Huang
  *   plushuang.tw@gmail.com
  *
  *  This library is free software; you can redistribute it and/or
@@ -80,6 +80,7 @@ struct UgtkApp
 //	UgRegistry      plugins;
 //	UgetPluginInfo* plugin;
 //	UgetTask        task;
+//	UgArrayPtr      nodes;
 //	void*           uri_hash;
 //	char*           config_dir;
 //	int             n_error;    // these n_xxxx increase by grow()
@@ -91,11 +92,12 @@ struct UgtkApp
 	UgtkSetting       setting;
 	UgtkScheduleState schedule_state;
 
+	// recent download settings
 	struct {
 		UgetNode*   infonode;
 		int         category_index;
 		int         saved;
-	} last;
+	} recent;
 	// status
 	gboolean        user_action;
 
@@ -109,6 +111,8 @@ struct UgtkApp
 		gchar*         text;
 		GRegex*        regex;
 		gboolean       processing;
+		// media website
+		gboolean       media_website;
 	} clipboard;
 
 	// dialogs
@@ -195,6 +199,8 @@ void  ugtk_app_quit (UgtkApp* app);
 // set UgtkSetting from/to UgtkApp
 void  ugtk_app_get_window_setting (UgtkApp* app, UgtkSetting* setting);
 void  ugtk_app_set_window_setting (UgtkApp* app, UgtkSetting* setting);
+void  ugtk_app_get_column_setting (UgtkApp* app, UgtkSetting* setting);
+void  ugtk_app_set_column_setting (UgtkApp* app, UgtkSetting* setting);
 void  ugtk_app_set_plugin_setting (UgtkApp* app, UgtkSetting* setting);
 void  ugtk_app_set_other_setting (UgtkApp* app, UgtkSetting* setting);
 void  ugtk_app_set_menu_setting (UgtkApp* app, UgtkSetting* setting);
@@ -216,7 +222,7 @@ void  ugtk_app_delete_download (UgtkApp* app, gboolean delete_files);
 void  ugtk_app_edit_category (UgtkApp* app);
 void  ugtk_app_edit_download (UgtkApp* app);
 // queue/pause
-void  ugtk_app_queue_download (UgtkApp* app);
+void  ugtk_app_queue_download (UgtkApp* app, gboolean keep_active);
 void  ugtk_app_pause_download (UgtkApp* app);
 void  ugtk_app_switch_download_state (UgtkApp* app);
 // move selected node
