@@ -1,6 +1,6 @@
 /*
  *
- *   Copyright (C) 2005-2015 by C.H. Huang
+ *   Copyright (C) 2005-2016 by C.H. Huang
  *   plushuang.tw@gmail.com
  *
  *  This library is free software; you can redistribute it and/or
@@ -81,13 +81,13 @@ void  ugtk_download_form_init (UgtkDownloadForm* dform, UgtkProxyForm* proxy, Gt
 
 static void ugtk_download_form_init_page1 (UgtkDownloadForm* dform, UgtkProxyForm* proxy)
 {
-	GtkWidget*	widget;
-	GtkGrid*	top_grid;
-	GtkGrid*	grid;
-	GtkWidget*	frame;
+	GtkWidget*  widget;
+	GtkGrid*    top_grid;
+	GtkGrid*    grid;
+	GtkWidget*  frame;
 	GtkBox*	    top_vbox;
-	GtkWidget*	vbox;
-	GtkWidget*	hbox;
+	GtkWidget*  vbox;
+	GtkWidget*  hbox;
 
 	dform->page1 = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
 	top_vbox = (GtkBox*) dform->page1;
@@ -282,8 +282,8 @@ static void ugtk_download_form_init_page1 (UgtkDownloadForm* dform, UgtkProxyFor
 
 static void ugtk_download_form_init_page2 (UgtkDownloadForm* dform)
 {
-	GtkWidget*	widget;
-	GtkGrid*	grid;
+	GtkWidget*  widget;
+	GtkGrid*    grid;
 
 	dform->page2 = gtk_grid_new ();
 	grid = (GtkGrid*) dform->page2;
@@ -410,7 +410,7 @@ static void ugtk_download_form_init_page2 (UgtkDownloadForm* dform)
 	// spin - Max upload speed
 	widget = gtk_spin_button_new_with_range (0, 99999999, 1);
 	gtk_entry_set_activates_default (GTK_ENTRY (widget), TRUE);
-	gtk_entry_set_width_chars (GTK_ENTRY (widget), 8);
+//	gtk_entry_set_width_chars (GTK_ENTRY (widget), 8);
 	g_object_set (widget, "margin", 1, NULL);
 	gtk_grid_attach (grid, widget, 2, 5, 1, 1);
 	dform->spin_upload_speed = (GtkSpinButton*) widget;
@@ -428,7 +428,7 @@ static void ugtk_download_form_init_page2 (UgtkDownloadForm* dform)
 	// spin - Max download speed
 	widget = gtk_spin_button_new_with_range (0, 99999999, 1);
 	gtk_entry_set_activates_default (GTK_ENTRY (widget), TRUE);
-	gtk_entry_set_width_chars (GTK_ENTRY (widget), 8);
+//	gtk_entry_set_width_chars (GTK_ENTRY (widget), 8);
 	g_object_set (widget, "margin", 1, NULL);
 	gtk_grid_attach (grid, widget, 2, 6, 1, 1);
 	dform->spin_download_speed = (GtkSpinButton*) widget;
@@ -440,6 +440,7 @@ static void ugtk_download_form_init_page2 (UgtkDownloadForm* dform)
 
 	// Retrieve timestamp
 	widget = gtk_check_button_new_with_label (_("Retrieve timestamp"));
+	g_object_set (widget, "margin-top", 3, "margin-bottom", 1, NULL);
 	gtk_grid_attach (grid, widget, 0, 7, 3, 1);
 	dform->timestamp = (GtkToggleButton*) widget;
 }
