@@ -1,6 +1,6 @@
 /*
  *
- *   Copyright (C) 2012-2015 by C.H. Huang
+ *   Copyright (C) 2012-2016 by C.H. Huang
  *   plushuang.tw@gmail.com
  *
  *  This library is free software; you can redistribute it and/or
@@ -228,9 +228,9 @@ static void ugtk_menubar_edit_init (UgtkMenubar* menubar)
 	gtk_menu_shell_append ((GtkMenuShell*)menu, menu_item);
 	menubar->edit.skip_existing = menu_item;
 
-	menu_item = gtk_check_menu_item_new_with_mnemonic (_("Apply recently download settings"));
+	menu_item = gtk_check_menu_item_new_with_mnemonic (_("Apply recent download settings"));
 	gtk_menu_shell_append ((GtkMenuShell*)menu, menu_item);
-	menubar->edit.apply_recently = menu_item;
+	menubar->edit.apply_recent = menu_item;
 
 	gtk_menu_shell_append ((GtkMenuShell*)menu, gtk_separator_menu_item_new());
 
@@ -377,11 +377,11 @@ static void ugtk_menubar_view_init (UgtkMenubar* menubar)
 	gtk_menu_item_set_submenu ((GtkMenuItem*) menu_item, submenu);
 	gtk_menu_shell_append ((GtkMenuShell*)menu, menu_item);
 	menubar->view.columns.self = submenu;
-	// Download Columns - Completed
+	// Download Columns - Complete
 	menu_item = gtk_check_menu_item_new_with_mnemonic (_("_Complete"));
 	gtk_menu_shell_append ((GtkMenuShell*) submenu, menu_item);
 	gtk_check_menu_item_set_active ((GtkCheckMenuItem*) menu_item, TRUE);
-	menubar->view.columns.completed = menu_item;
+	menubar->view.columns.complete = menu_item;
 	// Download Columns - Total
 	menu_item = gtk_check_menu_item_new_with_mnemonic (_("_Size"));
 	gtk_menu_shell_append ((GtkMenuShell*) submenu, menu_item);
