@@ -1,6 +1,6 @@
 /*
  *
- *   Copyright (C) 2012-2015 by C.H. Huang
+ *   Copyright (C) 2012-2016 by C.H. Huang
  *   plushuang.tw@gmail.com
  *
  *  This library is free software; you can redistribute it and/or
@@ -693,7 +693,7 @@ static void on_complete_column_clicked (GtkTreeViewColumn *column, UgtkTraveler*
 
 static void on_size_column_clicked (GtkTreeViewColumn *column, UgtkTraveler* traveler)
 {
-	ugtk_tree_view_column_clicked (column, UGTK_NODE_COLUMN_SIZE, traveler);
+	ugtk_tree_view_column_clicked (column, UGTK_NODE_COLUMN_TOTAL, traveler);
 }
 
 static void on_percent_column_clicked (GtkTreeViewColumn *column, UgtkTraveler* traveler)
